import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

from scipy import interpolate

import moving_average

pi=math.pi
pi2=2*pi
c=3.0e8
eps0=8.85e-12
mu0=4.0e-7*pi

def Ra_Ca_balmain(fpe, fce, freq, La, raa):
    X=(fpe/freq)**2
    Y=fce/freq

    eps1=1.0-X/(1.0-Y*Y)
    eps3=1.0-X

    omega=pi2*freq

    #----- |a| parameter for Balmain impedance
    av=np.sqrt(abs(eps1/eps3))

    coef=av/(omega*pi2*eps0*eps1*La)

    # Ra for dipole = double of that for a monopole
    Rb=2.0*coef*(np.log(La/raa)-1.0-np.log(np.sqrt(1.0+av*av)/2.0))
    Xb=2.0*coef*np.arctan(av)

    return Rb, Xb

def Ra_wang_bell(fpe, fce, freq, La, raa):
    X=(fpe/freq)**2
    Y=fce/freq

    beta=pi2*freq/c
    Z0=377.0
    eps_0=1.0-X

    denom=Y*Y-1.0
    eps_s=X/denom
    alpha=Y/np.sqrt(denom)
        
    xi=(La*beta/alpha)*np.sqrt(eps_s)

    Rwb=Z0/(pi*alpha*np.sqrt(abs(eps_0))*xi) \
          *(np.log(2.0*La/(raa*alpha))-1.0-0.25*xi*xi+(Y+1)**1.5*xi**3/12.0)

    return Rwb

#Song et al formula for Ra
def Ra_song_coefficient(fpe, fce, freq):
    mu0=4.0e-7*math.pi
    c=3.0e8

    X=(fpe/freq)**2
    Y=fce/freq

    denom=1.0-Y*Y
    eps1=1.0-X/denom
    eps2=X*Y/denom
    eps3=1.0-X

    sinth2=1.0-4.0/Y**2
    q=(eps1**2-eps2**2-eps1*eps3)**2*sinth2**2+16.0*(eps2*eps3/Y)**2
    Cx=0.5*X*(X-eps3)*sinth2-X*(eps1*sinth2+4.0*eps3/Y**2)+eps1*eps3
    Cy=eps2*(X*sinth2-eps3)
    Ra_coef=48.0*pi**3*mu0*freq**2*fpe*(Cx*Cx+Cy*Cy)*sinth2/(c*fce*q)

    return Ra_coef

#------------------------------------------------------------------------------
# calculate normalized data points in each frequency - Xa or frequency - Ra bin
def number_data_points(N, ybin, fres, bwd, CaXa, Ra, corx):
    #-- for dot by dot plots of data points in each bin: y-axis
    y=np.zeros((2,ybin+1))
    yCX=np.zeros(ybin)
    yRa=np.zeros(ybin)

    fsmin=min(fres)
    fsmax=max(fres)

    xbin=int((fsmax-fsmin)/bwd)
    x=[0]*(xbin+1)
    xfr=[0]*xbin
    for k in range(xbin+1):
        x[k]=fsmin+float(k)*bwd
        if k > 0:
            xfr[k-1]=0.5*(x[k-1]+x[k])

    #--- determine y min & max
    CXmin=min(CaXa)
    CXmax=max(CaXa)
    Ramin=min(Ra)
    Ramax=max(Ra)
    dCX=(CXmax-CXmin)/ybin
    dRa=(Ramax-Ramin)/ybin

    #--- bin size and center of bins for Ca, Ra
    for k in range(ybin+1):
        y[0][k]=CXmin+float(k)*dCX
        y[1][k]=Ramin+float(k)*dRa
        if k > 0:
            yCX[k-1]=0.5*(y[0][k-1]+y[0][k])
            if corx == 'Xa':
                yCX[k-1]=10.0**yCX[k-1]

            yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
            yRa[k-1]=10.0**yRa[k-1]

    # --- calculate number of Ca data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,CaXa,fres,statistic='count', \
        bins=[x,y[0]])
    a=counts.statistic  # number of data points for Ca
    a=a.transpose()

    rdl=len(a[0])+1-N
    CXb2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        CXb2d[k]=moving_average.data_points_running_average(a[k], N)
    CXb2d=CXb2d.transpose()

    # --- calculate number of Ra data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,Ra,fres,statistic='count', \
        bins=[x,y[1]])
    a=counts.statistic  # number of data points for Ra
    a=a.transpose()

    rdl=len(a[0])+1-N
    Rab2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        Rab2d[k]=moving_average.data_points_running_average(a[k], N)
    Rab2d=Rab2d.transpose()

    return rdl, xfr, yCX, yRa, CXb2d, Rab2d
#------------------------------------------------------------------------------

#-------------------- plot Ra, Ca, and Prad only for Y+ plus Y- ---------------------------
def plot_RaCa_byfcefpe(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, corx):

    if inpflag !='In':
        print("Empirical electron density model only valid inside plasmasphere!")
        print('Change flag to inside the plasmasphere')
        inpflag='In'

    datatpy=input('derived or song or balmain or wang? ')

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        datafile=fdir+'All-Bands_InOut_data.dat'
    else:
        datafile=fdir+'/All-Bands_InOut_data.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    fcefpe='/media/jtu/diska/dsx/export/DSX_orbit/orbit_fce_fpe.dat'
    fce_tpy=input('Use B-field from orbit data file or T96 model? orb or t96 ')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,1.8),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    if datatpy=='derived':
        nop=3
        xL=[0.10,0.10,0.10]
        yB=[0.68,0.44,0.20]
        wh=[0.48,0.48,0.48]
        ht=[0.22,0.22,0.22]

        txtsize=2.3
    else:
        nop=2
        xL=[0.10,0.10]
        yB=[0.57,0.22]
        wh=[0.52,0.52]
        ht=[0.32,0.32]

        txtsize=2.5

    axes=[]
    for i in range(nop):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for i in range(nop):
        for ax in ['left','bottom','right','top']:
            axes[i].spines[ax].set_linewidth(0.3)

        axes[i].tick_params(axis='both',which='major',length=1.3,width=0.4, \
            pad=1,labelsize=txtsize,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 8080:
        sL1=sL1+r' $V_1 >= 80$ V'

    LL=[0.0375, 0.3015, 0.1885, 2.1515]

    date=[]
    ut=[]
    CaXa=[]
    Ra=[]
    Pa=[]
    fres=[]

    freq1=[]
    freq2=[]
    Ia1=[]
    Ia2=[]
    Va1=[]
    Va2=[]

    La=41.0
    raa=0.15e-3
    qe=1.6022e-19
    me=9.1e-31
    eps0=8.85e-12
    coef=4.0*pi*pi*me*eps0/(qe*qe)

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpl != 1:
            print('Out of the plasmasphere:','{:2d}'.format(inpl)+' '+datet+' '+a[1])
            diff=diff+1
            continue

        fqp=float(a[5])
        fqm=float(a[16])
        Ra1=float(a[9])     #Ra in Ohm
        Ra2=float(a[20])

        if(L1 < 0.06 and (fqp>30.1e3 or fqm>30.1e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.2e3 or fqm>10.2e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 3.9e3 or fqm >3.9e3)):
            diff=diff+1
            continue

        freq1.append(fqp)
        freq2.append(fqm)
        Iap=float(a[8])*1.0e-3
        Iam=float(a[19])*1.0e-3
        Ia1.append(Iap)      # Ia in A
        Ia2.append(Iam)      # Ia in A

        Ca1=float(a[11])     # antenna Ca in pF
        Ca2=float(a[22])

        #Ra1=Ra1+Ra2
        #Ca1=Ca1*Ca2/(Ca1+Ca2)

        date.append(datet)
        ut.append(float(a[1][0:2])*60.0+float(a[1][3:5])+float(a[1][6:8])/60.0) #Ut in min

        # measured antenna reactance two branches in Ohm
        Xa1=1.0e12/(pi2*fqp*Ca1)
        Xa2=1.0e12/(pi2*fqm*Ca2)

        if corx == 'Ca':
            CaXa.append(Ca1)               # Ca in pF
        else:
            CaXa.append(np.log10(Xa1+Xa2))     # logarithm of -Xa in Ohm

        Ra.append(np.log10(Ra1+Ra2))           #logarithm of Ra in Ohm

        Vap=float(a[6])
        Vam=float(a[17])
        Va1.append(Vap)
        Va2.append(Vam)

        Pa.append(np.log10(0.5*( Vap**2*Ra1/(Ra1**2+Xa1**2) \
                                +Vam**2*Ra2/(Ra2*82+Xa2**2))))

        fres.append(fqp/1.0e3)  # frequency as x-axis in kHz

    #-----------------------------------------------------------
    #end 'for loop' for input antenna parameters
    #-----------------------------------------------------------
    if diff > 0:
        print('\n',diff,'data out of',nrecd,'not used. Used data: ', nrecd-diff)

    #input fce_orb and fpe_orb from file orbit_fce_fpe.dat, 1 min resolution
    fid=open(fcefpe, 'r')
    fid.readline()
    ffcefpe=fid.readlines()
    fid.close()

    nfcefpe=len(ffcefpe)

    ndata=len(ut)

    Xa_song=[] #np.zeros(ndata)
    Ra_song=[] #np.zeros(ndata)
    #Pa_song=[]
    Xa_balm=[] #np.zeros(ndata)
    Ra_balm=[] #np.zeros(ndata)
    #Pa_balm=[]
    Ra_wang=[] #np.zeros(ndata)

    kk=0
    fress=[]
    jorb=0
    for i in range(ndata):
        if i == 0 or (i>0 and date[i] != date[i-1]):

    #-------------------------------------------------------------------
    # intepolate fce and fpe to UT of Ca/Ra data file for invidual date
            k=-1

            time_orb=[]
            fpe_orb=[]
            fce_orb=[]

            for j in range(jorb,nfcefpe):
                a=ffcefpe[j].split()

                if a[0] != date[i]:
                    if k < 0:
                        continue
                    else:
                        break

                time_orb.append(float(a[1]))
                fpe_orb.append(float(a[11]))
                if fce_tpy=='orb':
                    fce_orb.append(float(a[12]))
                else:
                    fce_orb.append(float(a[10]))

                k=1

            jorb=j

            # intepolation function for fce and fpe for this day
            fce_func=interpolate.interp1d(time_orb, fce_orb, copy=False, \
                fill_value='extrapolate')
            fpe_func=interpolate.interp1d(time_orb, fpe_orb, copy=False, \
                fill_value='extrapolate')

        fce=fce_func(ut[i])
        fpe=fpe_func(ut[i])

        # remove data points with invalid fce (based on empirical model given B-field)
        if fce <= 2.0*freq1[i]:
            print('fce <= 2 f_r: ',('{:12.4f}'*2).format(fce, freq1[i]))
            continue
        if fce >= fpe:
            print('fce >= fpe: ',('{:12.4f}'*2).format(fce, fpe))
            continue

#-------------------------------------------------------------------

#-------------------------------------------------------------------
        # Song et al [2007] Xa & Song et al [2021] Ra. 
        # scale to 3-wire in parallel: Xa and Ra divided by 2.2
        Omega1=pi2*freq1[i]
        N0=coef*fpe**2
        aa=pi*Omega1*La
        Xa_s=1.0/(4.0*aa*eps0)*(np.log(Ia1[i]/(aa*qe*N0*raa*raa)+2.0)-1.0)/2.2
        Xa_s=Xa_s+1.0/(4.0*aa*eps0)*(np.log(Ia2[i]/(aa*qe*N0*raa*raa)+2.0)-1.0)/2.2
        #Ra_s=3.9e-13*fce**3*fpe*La*La/(freq1[i]*freq1[i])/2.2
        Ra_s=Ra_song_coefficient(fpe, fce, freq1[i])*La**2/2.2
        #Pa_song.append(np.log10(0.5*Va[i]**2*Ra_s/(Ra_s**2+Xa_s**2)))
        Xa_song.append(np.log10(Xa_s))
        Ra_song.append(np.log10(Ra_s))

        # Bailmain [1964] Xa and Ra of dipole
        Ra_b, Xa_b=Ra_Ca_balmain(fpe, fce, freq1[i], La, raa)
        #Pa_balm.append(np.log10(0.5*Va[i]**2*Ra_b/(Ra_b**2+Xa_b**2)))

        Xa_balm.append(np.log10(Xa_b*3000))  #multiply Xa_b by 3000 but not scaled by 2.2
        Ra_balm.append(np.log10(Ra_b))

        # Wand & Bell Ra
        Ra_wang.append(np.log10(Ra_wang_bell(fpe, fce, freq1[i], La, raa)))

        fress.append(freq1[i]/1.0e3)  # frequency as x-axis in kHz

        kk=kk+1
#-------------------------------------------------------------------

    print('Number of invalid fce data removed: ', nrecd-diff-kk,'\n')

    fsmin=min(fress)
    fsmax=max(fress)

    if bname == 'All-Bands':
        xscale='log'
        if corx == 'Ca':
            yscale='linear'
        else:
            yscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        yscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        yscale='linear'
        fmin=min(fress)
        fmax=max(fress)

    # --- calculate dipole antenna capacitance and resistance in vacuum
    xbin=int(fsmax-fsmin)
    freq=[0]*xbin
    Rvac=[0]*xbin
    Cvac=[0]*xbin
    Xvac=[0]*xbin

    La=82.0
    raa=0.2e-3
    for j in range(xbin):
        freq[j]=fsmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        Omega=pi2*freq[j]*1.0e3
        Rvac[j]=80.0*(pi*La/wlen)**2      # vacuum dipole resistance
        k0=Omega/3.0e8
        Xvac[j]=120.0*(np.log(La/(2.0*raa))-1.0)/np.tan(k0*La/2.0)
        Cvac[j]=1.0e12/(Omega*Xvac[j])    # vacuum dipole Capacitance

        #dipole Ca, Xa, and Ra for 3 wires
        Cvac[j]=Cvac[j]*3.0
        Xvac[j]=Xvac[j]
        Rvac[j]=Rvac[j]*1.0e5 #in ohm scaled by 1x10^5 to fit the figure

    print('Min & max vacuum Ca:',("{:8.2f}"*2).format(min(Cvac),max(Cvac)))
    print('Min & max vacuum Ra*10^5:',("{:8.2f}"*2).format(min(Rvac),max(Rvac)))

    #---------------------------------------------------------------------------------
    N=8   # running average window width
    ybin=100 # for 2D binning not for 1D

    # plot number of derived, or Song et al or Balmain [19640 data points in each bin, 
    # onece for yCa (or yRa or ypw) points at given frequency xfr[j]. The data points 
    # number is normalized by maximum at each frequency
    #
    #--------- Xa and Ra in log10 freq in kHz ----------------------------------------
    if datatpy == 'derived':
        rdl, xfr, yCX, yRa, CXb2d, Rab2d = number_data_points(N, ybin, fres, bwd, \
            CaXa, Ra, corx)  #note CaXa/Ra/fres has different size from fress
        rdl, xfr, yCX, yPa, CXb2d, Pab2d = number_data_points(N, ybin, fres, bwd, \
            CaXa, Pa, corx)  # this call to get number of data points for Pa
        print(datatpy)
    elif datatpy == 'song':
        rdl, xfr, yCX, yRa, CXb2d, Rab2d = number_data_points(N, ybin, fress, bwd, \
            Xa_song, Ra_song, corx)
        #rdl, xfr, yCX, yPa, CXb2d, Pab2d = number_data_points(N, ybin, fress, bwd, \
        #    Xa_song, Pa_song, corx)  # this call to get number of data points for Pa
        print(datatpy)
    elif datatpy == 'balmain':
        rdl, xfr, yCX, yRa, CXb2d, Rab2d = number_data_points(N, ybin, fress, bwd, \
            Xa_balm, Ra_balm, corx) #Xa_balm is log10(Xa*3000)
        #rdl, xfr, yCX, yPa, CXb2d, Pab2d = number_data_points(N, ybin, fress, bwd, \
        #    Xa_balm, Pa_balm, corx)  # this call to get number of data points for Pa
        print(datatpy)
    else:
        rdl, xfr, yCX, yRa, CXb2d, Rab2d = number_data_points(N, ybin, fress, bwd, \
            Xa_balm, Ra_wang, corx)
        print(datatpy)

    #--- plot scatter data points at frequency xfr[j] (kHz): the center of frequency bin
    sc=[0]*3
    for j in range(len(xfr[0:rdl])):
        xx=[xfr[j]]*ybin

        CXbmax=max(CXb2d[j])  #maximum number of data points in this frequency bin
        if CXbmax > 0:
            sc[0]=axes[0].scatter(xx,yCX,c=CXb2d[j]/CXbmax,cmap=colormp,s=0.1,marker='s')

        Rabmax=max(Rab2d[j])
        if Rabmax > 0:
            sc[1]=axes[1].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1,marker='s')

        if datatpy == 'derived':
            Pabmax=max(Pab2d[j])
            if Pabmax > 0:
                sc[2]=axes[2].scatter(xx,yPa,c=Pab2d[j]/Pabmax,cmap=colormp, \
                    s=0.1,marker='s')
    #---------------------------------------------------------------------------------------

#----- scaled RPI capacitance
    Ld=41.0     # one brahch length
    rd=0.15e-3
    LR=125.0    # 125 m -branche
    rR=0.20e-3
    fRPI=[32.6,34.0]
    CXRPI=[430e-12,480e-12]
    a1=(np.exp(4.0*pi*eps0*LR/(1.5*CXRPI[0])+1.0)-2.0)*LR*rR*rR
    a2=(np.exp(4.0*pi*eps0*LR/(1.5*CXRPI[1])+1.0)-2.0)*LR*rR*rR
    # one 125 m branch capacitance of RPI = CXRPI * 3/2.
    # and three wires/two branches by another factor 3/2
    CXRPI[0]=3.0/2.0*Ld/LR*3.0/2.0*CXRPI[0]*(np.log(a1/(LR*rR*rR)+2.0)-1.0) \
             /(np.log(a1/(Ld*rd*rd)+2.0)-1.0)
    CXRPI[1]=3.0/2.0*Ld/LR*3.0/2.0*CXRPI[1]*(np.log(a2/(LR*rR*rR)+2.0)-1.0) \
             /(np.log(a2/(Ld*rd*rd)+2.0)-1.0)
    #----- completed scaling RPI capacitance
    RaRPI=[250,500]

    #----- scale Tu et al [2008] reactance & resistance to DSX
    LT=50.0 #one branch length
    rT=0.2
    NT=500.0e6
    Nd=2000.0e6
    fTu=[30.0]
    Omega=pi*fTu[0]*1.0e3
    CXTu=[3486.2/2.0] # one branch reactance
    a1=(np.exp(4.0*pi*Omega*eps0*LT*CXTu[0]+1.0)-2.0)*LT*rT*rT*NT
    #scaled to DSX reactance for two branches/3 wires in Ohm
    CXTu[0]=2.0/3.0*CXTu[0]*LT/Ld*(np.log(a1/(Nd*Ld*rd*rd)+2.0)-1.0) \
            /(np.log(a1/(NT*LT*rT*rT)+2.0)-1.0)
    RaTu=[0]
    RaTu[0]=CXTu[0]/np.tan(88.59*pi/180.0)
    #----- completed scaling Tu et al reactance & resistance

    CXRPI[0]=1.0e-3/(pi2*fRPI[0]*CXRPI[0]) #reactance in kilo Ohm
    CXRPI[1]=1.0e-3/(pi2*fRPI[1]*CXRPI[1])

    #axes[0].scatter(fres, Xa_song, s=0.04, marker='+',color='black', edgecolors=None, \
    #    label='Song et al [2007]')
    #axes[0].scatter(fTu,CXTu, s=1, marker='^', color='magenta', label='Tu et al [2008]')
    #axes[0].plot(fres,Xa_balm, lw=0.3, color='black', label='Balmain [1964]')
    #axes[0].scatter(fres, Xa_balm*100, s=0.3, marker='.', color='black', label='Balmain [1964]')

    # --- y-axis label
    if corx == 'Ca':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=txtsize,labelpad=0.7)
    else:
        axes[0].set_ylabel('$-X_a$ (Ohm)',fontsize=txtsize,labelpad=0.7)

   # ---- plot theoretical Ra with given fpe, fce and I0 vs f (Song et al., 2021) ---
    #axes[1].scatter(fres,Ra_song, marker='+',s=0.04, color='black', edgecolors=None, \
    #    label='Song et al. [2021]')

    #axes[1].scatter(fTu,RaTu, s=1, marker='^', color='magenta', label='Tu et al [2008]')
    #axes[1].scatter(fres,Ra_song, s=0.3, marker='.', color='red',label='Song et al [2007]')
    #axes[1].scatter(fres,Ra_balm, s=0.3, marker='.', color='black',label='Balmain [1964]')

    #----- set axis properties for Ca or Xa------
    axes[0].set_xlim(fmin,fmax)
    axes[0].set_ylim(6e3,7e5)
    axes[0].set_xscale(xscale)
    axes[0].set_yscale(yscale)
    axes[0].set_xticklabels([])

    axes[1].set_xlim(fmin,fmax)
    axes[1].set_ylim(30,1e5)
    axes[1].set_xscale(xscale)
    axes[1].set_yscale('log')
    axes[1].set_ylabel('$R_a$ (Ohm)',fontsize=txtsize,labelpad=0.7)

    if datatpy == 'derived':
        axes[0].plot(freq, Xvac, lw=0.3, color='black', ls='dashed',\
            label='Dipole in Vacuum')
        axes[1].plot(freq, Rvac, lw=0.3, color='black', ls='dashed',\
            label='Dipole in Vacuum')
        axes[1].text(0.34,0.18,'Multipled by $10^5$',fontsize=txtsize,ha='center', \
            va='bottom', transform=axes[1].transAxes,color='black',rotation=14)

        axs=axes[2]
        scc=sc[2]
        yloc=-0.47
        dyloc=0.0

        axes[1].set_xticklabels([])
        axes[2].set_xlim(fmin,fmax)
        #axes[2].set_ylim(30,1e5)
        axes[2].set_xscale(xscale)
        axes[2].set_yscale('log')
        axes[2].set_ylabel('$P_a$ (W)',fontsize=txtsize,labelpad=0.7)
        axes[2].set_xlabel('Resonance Frequency (kHz)',fontsize=txtsize, labelpad=0.7)

        axes[2].set_xticks([2.5,5,10,20,30,40])
        axes[2].set_xticklabels(['2.5','5','10','20','30','40'])
    else:
        axes[1].set_xlabel('Resonance Frequency (kHz)',fontsize=txtsize, labelpad=0.7)
        axes[1].set_xticks([2.5,5,10,20,30,40])
        axes[1].set_xticklabels(['2.5','5','10','20','30','40'])

        axs=axes[1]
        scc=sc[1]
        yloc=-0.38
        dyloc=0.02

    XX=[160e3]
    freq[0]=fsmin
    for j in range(1, xbin):
        freq[j]=fsmin+float(j)
        XX.append(XX[0]*(fsmin/freq[j])**0.91)

    axes[0].plot(freq, XX, ls='dashed', lw=0.3, color='blue', \
        label='Fitting to Measured $X_a$ ')

    RR=[3.0e4]
    for j in range(1, xbin):
        RR.append(RR[0]*(fsmin/freq[j])**1.7)

    axes[1].plot(freq, RR, ls='dashed', lw=0.3, color='blue', \
        label='Fitting to Measured $R_a$')

    for j in range(2):
        legd=axes[j].legend(loc='upper center',fontsize=2.3,bbox_to_anchor=(0.65,0.99), \
            frameon=False)
        for text in legd.get_texts():
            text.set_color('blue')

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' Inside Nominal Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' Outside Plasmasphere'
    else:
        txt=sL1+' All Regions'

    xloc=0.5
    if datatpy=='derived':
        axes[0].text(xloc,1.115-dyloc,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
            ha='center',va='bottom',transform=axes[0].transAxes)
    elif datatpy == 'song':
        txt='Song et al [2007] & Song et al [2021]'
        axes[0].text(0.95,0.92,'(a)',fontsize=txtsize,ha='center',va='bottom', \
            transform=axes[0].transAxes)
        axes[1].text(0.95,0.92,'(b)',fontsize=txtsize,ha='center',va='bottom', \
            transform=axes[1].transAxes)
    elif datatpy == 'balmain':
        txt=r'Balmain [1964] $X_a$ multiplied by 3000'
        axes[0].text(0.95,0.92,'(c)',fontsize=txtsize,ha='center',va='bottom', \
            transform=axes[0].transAxes)
        axes[1].text(0.95,0.92,'(d)',fontsize=txtsize,ha='center',va='bottom', \
            transform=axes[1].transAxes)
    elif datatpy == 'wang':
        txt='Wang and Bell [1969]'

    axes[0].text(xloc,1.02,txt,fontsize=txtsize,ha='center',va='bottom', \
        transform=axes[0].transAxes)

    # ---- color bar ----
    xloc=0.085
    axins = inset_axes(axs, width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(xloc,yloc,1,1), bbox_transform=axs.transAxes)
    cbar=fig.colorbar(scc, cax=axins, shrink=0.9, pad=0.04, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar
