import numpy as np

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure

from matplotlib.ticker import AutoMinorLocator

from scipy import interpolate

#-------------------------------------------------------------------------------------
#------------ Calculate Q-factor -----------------------------------------------------
#-------------------------------------------------------------------------------------
def calc_Q(Va,freq):
    #performing interpolate to increase frequency resolution so as to increase accuracy
    #of the Q calculation
    maxfq=max(freq)
    fqin=np.linspace(min(freq), maxfq, len(freq)*100)
    fq=np.flipud(fqin)

    Va_interp=interpolate.interp1d(freq, Va, kind='linear')
    Va1=Va_interp(fq)

    ipeak=np.argmax(Va1)
    V70p=np.max(Va1)/np.sqrt(2)

    iL1=-1
    iL=-1
    iL2=-1
    #original order is from high to low frequency - needed to be reversed
    for i in range(ipeak, len(fq)):
        if Va1[i] > V70p:                       #  +            ipeak
            iL2=i                              #   +        
        elif Va1[i] == V70p:                    #     +         iL2
            iL=i                               #       +    ------- V70p (iL)
            break                              #         +     iL1
        else:                                  #          +
            iL1=i                              #            +  len(freq)
            break

    for i in range(ipeak, iL1):
        if Va1[i+1] >= Va1[i]:
            return -2.0                       #Va non monotonous dcrease

    if iL == -1:
        if iL2 >= ipeak and iL1 > iL2:
            fhL= (fq[iL1]*(Va1[iL2]-V70p)+fq[iL2]*(V70p-Va1[iL1])) \
                /(Va1[iL2]-Va1[iL1])
        else:
            return -1.0
    else:
        fhL=fq[iL]

    iH1=-1
    iH=-1
    iH2=-1
    for i in range(0, ipeak+1):
        if Va1[i] < V70p:                       #           +  ipeak
            iH1=i                              #          +
        elif Va1[i] == V70p:                    #        +     iH2
            iH=i                               #       +   --------V70p (iH)
            break                              #      +       iH1
        else:                                  #    +
            iH2=i                              #   +          0
            break

    for i in range(iH1,ipeak):
        if Va1[i+1] <= Va1[i]:
            return -2.0                       #Va non monotonous increase

    if iH == -1:
        if iH1 >= 0 and iH2 > iH1:
            fhH= (fq[iH1]*(Va1[iH2]-V70p)+fq[iH2]*(V70p-Va1[iH1])) \
                /(Va1[iH2]-Va1[iH1])
        else:
            return -1.0
    else:
        fhH=fq[iH]

    Q=freq[np.argmax(Va)]/(fhH-fhL)

    return Q

#####################################################################################
#####################################################################################
def plot_Vap_Vam(wd, canv, fig, toolbar, data_all, bname):

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(1.8,1.08),dpi=600)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=0.15
    yB=0.15
    wh=0.70
    ht=0.70

    #fig=plt.figure(figsize=(1.9, 1.5), dpi=400)
    axes=fig.add_axes([xL, yB, wh, ht])
    fig.add_axes(axes)

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        axes.spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(data_all[0][2][1]))+' H'
    if bname == 'All-Bands':
        sL1='All-Bands'

    nrecd=len(data_all)

    date=[]
    entv=[]
    entv3=[]
    Q=[]
    freq=[]
    Vap=[]
    Vam=[]
    prog=[]

    for k in range(nrecd):
        date.append(data_all[k][0])
        entv.append(data_all[k][2])
        entv3.append(data_all[k][3])
        Q.append(data_all[k][4])
        freq.append(data_all[k][5])
        Vap.append(data_all[k][6])
        Vam.append(data_all[k][10])
        prog.append(data_all[k][15])

    Va_p=[]
    Va_m=[]

    Va_p01=[]
    Va_m01=[]

    Va_p2=[]
    Va_m2=[]
     
    diff=0
    for k in range(0,nrecd):
        if k == 0 or (k > 0 and date[k][0:11] != date[k-1][0:11]):
            print(date[k])

        nfq=len(freq[k])

        fq=np.zeros(nfq)
        Va1=np.zeros(nfq)
    
        Va2=np.zeros(nfq)
    
        fq[0:nfq]=freq[k][0:nfq]

        Va1[0:nfq]=Vap[k][0:nfq] 
        Va2[0:nfq]=Vam[k][0:nfq] 

        nfsp=np.argmax(Va1)
        nfsm=np.argmax(Va2)

        #clculate Q using interpolated high resolution Va - freq data
        Qp=calc_Q(Va1,fq)
        Qm=calc_Q(Va2,fq)

        log0 = entv[k][1] != entv[k][4]
        log1 = entv[k][2] != entv[k][5]
        log2= Qp <= 0.0 or Qm <= 0.0
        log3= entv3[k][2] > 0.0 or entv3[k][5] > 0.0

        #exclude data with following conditions
        if log0 or log1:
            #if log0:
            #    print('L1 of two TATUs are different:              ' \
            #        +('{:8.4f}'*2).format(entv[k][1],entv[k][4])+' '+date[k] \
            #         +'{:5d}'.format(prog[k]))
            #elif log1:
            #    print('C1 of two TATUs are different:              ' \
            #        +('{:7.1f}'*2).format(entv[k][2],entv[k][5])+' '+date[k] \
            #        +'{:5d}'.format(prog[k]))
            Va_p01.append(Va1[nfsp])
            Va_m01.append(Va2[nfsm])
        elif log2:
            #print('Possibly no resonance on one or both TATUs: ' \
            #     +('{:10.2f}'*2).format(Qp,Qm)+' '+date[k]+'{:5d}' \
            #    .format(prog[k]))
            Va_p2.append(Va1[nfsp])
            Va_m2.append(Va1[nfsm])
            continue
        elif log3:
            #print('Third harmonic turned on one or both TATUs: ' \
            #    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k] \
            #    +'{:5d}'.format(prog[k]))

            diff=diff+1

            continue

        else:
            Va_p.append(Va1[nfsp])
            Va_m.append(Va1[nfsm])

    sL1=sL1+' '+date[0][0:11]+r'$-$'+date[len(date)][0:11]

# set axis, labels, color bar etc for left three panels
    axes.scatter(Va_m, Va_p, s=0.4, marker='.',color='red', edgecolors='none',\
        label='Symmetric setting')
    if len(Va_p01) > 0:
        axes.scatter(Va_m01, Va_p01, s=0.4, marker='.',color='blue', \
            edgecolors='none', label='Asymmetric setting')
    if len(Va_p2) > 0:
        axes.scatter(Va_m2, Va_p2, s=0.4, marker='.',color='black', \
            edgecolors='none', label='Undeterminable Q-factor')

    axes.legend(loc='upper center',fontsize=2.3,bbox_to_anchor=(0.1,0.99), \
            frameon=False)

    axes.set_xlim(0,60)
    axes.set_ylim(0,60)
    axes.tick_params(axis='both',which='major',length=1.3,width=0.4,pad=1, \
        labelsize=2.4,left=True,right=True,bottom=True,top=True)
    axes.tick_params(axis='both',which='minor',length=0.8,width=0.4,pad=1, \
        labelsize=0,left=True,right=True,bottom=True,top=True)
    axes.xaxis.set_minor_locator(AutoMinorLocator())
    axes.yaxis.set_minor_locator(AutoMinorLocator())

    axes.set_xlabel (r'$V_{a-}$ (V)', fontsize=2.4, labelpad=0.7)
    axes.set_ylabel (r'$V_{a+}$ (V)', fontsize=2.4, labelpad=0.7)
    axes.text(0.01,1.04,sL1,fontsize=2.5,ha='left',va='bottom', \
        transform=axes.transAxes)

    return canv, fig, toolbar

#####################################################################################
#####################################################################################
def plot_frp_frm(wd, canv, fig, toolbar, data_all, bname):

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(1.8,1),dpi=600)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=0.10
    yB=0.12
    wh=0.70
    ht=0.70

    #fig=plt.figure(figsize=(1.9, 1.5), dpi=400)
    axes=fig.add_axes([xL, yB, wh, ht])
    fig.add_axes(axes)

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        axes.spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(data_all[0][2][1]))+' H'
    if bname == 'All-Bands':
        sL1='All-Bands'

    nrecd=len(data_all)

    date=[]
    entv=[]
    entv3=[]
    Q=[]
    freq=[]
    Vap=[]
    phip=[]
    Vam=[]
    phim=[]
    inout=[]
    prog=[]

    fmins=[0]*nrecd
    fmaxs=[0]*nrecd

    for k in range(nrecd):
        date.append(data_all[k][0])
        entv.append(data_all[k][2])
        entv3.append(data_all[k][3])
        Q.append(data_all[k][4])
        freq.append(data_all[k][5])
        Vap.append(data_all[k][6])
        phip.append(data_all[k][8])
        Vam.append(data_all[k][10])
        phim.append(data_all[k][12])
        inout.append(data_all[k][14])
        prog.append(data_all[k][15])

        fmins[k]=min(data_all[k][5])
        fmaxs[k]=max(data_all[k][5])

    fmin=min(fmins)/1.0e3
    fmax=max(fmaxs)/1.0e3

    fres_p=[]
    fres_m=[]

    fres_p01=[]
    fres_m01=[]

    fres_p2=[]
    fres_m2=[]
     
    fres_p0=[]
    fres_m0=[]

    diff=0
    diff0=0
    kk=0
    for k in range(0,nrecd):
        #if k == 0 or (k > 0 and date[k][0:11] != date[k-1][0:11]):
        #    print(date[k])

        nfq=len(freq[k])

        fq=np.zeros(nfq)
        Va1=np.zeros(nfq)
        phi1=np.zeros(nfq)
    
        Va2=np.zeros(nfq)
        phi2=np.zeros(nfq)
    
        fq[0:nfq]=freq[k][0:nfq]

        Va1[0:nfq]=Vap[k][0:nfq] 
        Va2[0:nfq]=Vam[k][0:nfq] 

        phi1[0:nfq]=phip[k][0:nfq]
        phi2[0:nfq]=phim[k][0:nfq]

        nfsp=np.argmax(Va1)
        nfsm=np.argmax(Va2)

        if nfsp <=0 or nfsm <=0:
            continue

        #clculate Q using interpolated high resolution Va - freq data
        Qp=calc_Q(Va1,fq)
        Qm=calc_Q(Va2,fq)

        log0 = entv[k][1] != entv[k][4]
        log1 = entv[k][2] != entv[k][5]
        log2= Qp <= 0.0 or Qm <= 0.0
        log3= entv3[k][2] > 0.0 or entv3[k][5] > 0.0

        #exclude data with following conditions
        if log0 or log1:
            #if log0:
            #    print('L1 of two TATUs are different:              ' \
            #        +('{:8.4f}'*2).format(entv[k][1],entv[k][4])+' '+date[k] \
            #         +'{:5d}'.format(prog[k]))
            #elif log1:
            #    print('C1 of two TATUs are different:              ' \
            #        +('{:7.1f}'*2).format(entv[k][2],entv[k][5])+' '+date[k] \
            #        +'{:5d}'.format(prog[k]))
            fres_p01.append(fq[nfsp]/1e3)
            fres_m01.append(fq[nfsm]/1e3)
        elif log2:
            #print('Possibly no resonance on one or both TATUs: ' \
            #     +('{:10.2f}'*2).format(Qp,Qm)+' '+date[k]+'{:5d}' \
            #    .format(prog[k]))
            fres_p2.append(fq[nfsp]/1e3)
            fres_m2.append(fq[nfsm]/1e3)
        elif log3:
            #print('Third harmonic turned on one or both TATUs: ' \
            #    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k] \
            #    +'{:5d}'.format(prog[k]))

            continue

        else:
            if fq[nfsp] == fq[nfsm]:
                diff=diff+1
                fres_p.append(fq[nfsp]/1e3)
                fres_m.append(fq[nfsm]/1e3)

                if kk==0:
                    date_st=date[k][0:10]
                    kk=1

                date_end=date[k][0:10]

            elif fq[nfsp] != fq[nfsm]:
                diff0=diff0+1
                fres_p0.append(fq[nfsp]/1e3)
                fres_m0.append(fq[nfsm]/1e3)
                #print(date[k]+'{:5d}'.format(prog[k]))

    print('Total number of symmetrical setting cases: ', diff)
    print('Symmetrical setting cases with un-identical resonance frequencies: ', diff0)
 
    sL1=sL1+' '+date_st+r'$-$'+date_end

# set axis, labels, color bar etc for left three panels
    axes.scatter(fres_m, fres_p, s=0.5, marker='.',color='red', edgecolors='none',\
        label=r'Symmetric setting: $f_{r+} = f_{r-}$')
    if len(fres_p0) > 0:
        axes.scatter(fres_m0, fres_p0, s=0.4, marker='.',color='black', \
            edgecolors='none', label=r'Symmetric setting: $f_{r+} \neq f_{r-}$')
    #if len(fres_p01) > 0:
    #    axes.scatter(fres_m01, fres_p01, s=0.4, marker='.',color='blue', \
    #        edgecolors='none', label='Asymmetric setting')
    #if len(fres_p2) > 0:
    #    axes.scatter(fres_m2, fres_p2, s=0.4, marker='.',color='black', \
    #        edgecolors='none', label='Undeterminable Q-factor')

    axes.legend(loc='upper center',fontsize=2.7,bbox_to_anchor=(0.25,0.99), \
            frameon=False)

    axes.set_xlim(fmin,fmax)
    axes.set_ylim(fmin,fmax)
    axes.tick_params(axis='both',which='major',length=1.3,width=0.4,pad=1, \
        labelsize=2.5,left=True,right=True,bottom=True,top=True,direction='in')
    axes.tick_params(axis='both',which='minor',length=0.8,width=0.4,pad=1, \
        labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes.xaxis.set_minor_locator(AutoMinorLocator())
    axes.yaxis.set_minor_locator(AutoMinorLocator())

    axes.set_xlabel ('Resonance Frequency, +TATU (kHz)', fontsize=2.5, labelpad=0.7)
    axes.set_ylabel ('Resonance Frequency, -TATU (kHz)', fontsize=2.5, labelpad=0.7)
    axes.text(0.01,1.02,sL1,fontsize=2.7,ha='left',va='bottom', \
        transform=axes.transAxes)

    return canv, fig, toolbar

#####################################################################################
#####################################################################################
def plot_Iap_Iam(wd, canv, fig, toolbar, data_all, bname):

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(1.8,1),dpi=600)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=0.10
    yB=0.12
    wh=0.70
    ht=0.70

    #fig=plt.figure(figsize=(1.9, 1.5), dpi=400)
    axes=fig.add_axes([xL, yB, wh, ht])
    fig.add_axes(axes)

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        axes.spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(data_all[0][2][1]))+' H'
    if bname == 'All-Bands':
        sL1='All-Bands'

    nrecd=len(data_all)

    date=[]
    entv=[]
    entv3=[]
    Q=[]
    freq=[]
    Iap=[]
    Iam=[]

    fmins=[0]*nrecd
    fmaxs=[0]*nrecd

    for k in range(nrecd):
        date.append(data_all[k][0])
        entv.append(data_all[k][2])
        entv3.append(data_all[k][3])
        Q.append(data_all[k][4])
        freq.append(data_all[k][5])
        Iap.append(data_all[k][6])
        Iam.append(data_all[k][10])

        fmins[k]=min(data_all[k][5])
        fmaxs[k]=max(data_all[k][5])

    Ia_p=[]
    Ia_m=[]

    Ia_p01=[]
    Ia_m01=[]

    Ia_p2=[]
    Ia_m2=[]
     
    Ia_p0=[]
    Ia_m0=[]
     
    diff=0
    diff0=0
    kk=0
    for k in range(0,nrecd):
        if k == 0 or (k > 0 and date[k][0:11] != date[k-1][0:11]):
            print(date[k])

        nfq=len(freq[k])

        fq=np.zeros(nfq)
        Ia1=np.zeros(nfq)
    
        Ia2=np.zeros(nfq)
    
        fq[0:nfq]=freq[k][0:nfq]

        Ia1[0:nfq]=Iap[k][0:nfq] 
        Ia2[0:nfq]=Iam[k][0:nfq] 

        nfsp=np.argmax(Ia1)
        nfsm=np.argmax(Ia2)

        if nfsp <=0 or nfsm <=0:
            continue

        #clculate Q using interpolated high resolution Va - freq data
        Qp=calc_Q(Ia1,fq)
        Qm=calc_Q(Ia2,fq)

        log0 = entv[k][1] != entv[k][4]
        log1 = entv[k][2] != entv[k][5]
        log2= Qp <= 0.0 or Qm <= 0.0
        log3= entv3[k][2] > 0.0 or entv3[k][5] > 0.0

        #exclude data with following conditions
        if log0 or log1:
            #if log0:
            #    print('L1 of two TATUs are different:              ' \
            #        +('{:8.4f}'*2).format(entv[k][1],entv[k][4])+' '+date[k] \
            #         +'{:5d}'.format(prog[k]))
            #elif log1:
            #    print('C1 of two TATUs are different:              ' \
            #        +('{:7.1f}'*2).format(entv[k][2],entv[k][5])+' '+date[k] \
            #        +'{:5d}'.format(prog[k]))
            Ia_p01.append(fq[nfsp]/1e3)
            Ia_m01.append(fq[nfsm]/1e3)
        elif log2:
            #print('Possibly no resonance on one or both TATUs: ' \
            #     +('{:10.2f}'*2).format(Qp,Qm)+' '+date[k]+'{:5d}' \
            #    .format(prog[k]))
            Ia_p2.append(fq[nfsp]/1e3)
            Ia_m2.append(fq[nfsm]/1e3)
        elif log3:
            #print('Third harmonic turned on one or both TATUs: ' \
            #    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k])

            continue

        else:
            if nfsp == nfsm:
                diff=diff+1
                Ia_p.append(fq[nfsp]/1e3)
                Ia_m.append(fq[nfsm]/1e3)

                if kk==0:
                    date_st=date[k][0:10]
                    kk=1

                date_end=date[k][0:10]
            else:
                diff0=diff0+1
                Ia_p0.append(fq[nfsp]/1e3)
                Ia_m0.append(fq[nfsm]/1e3)

    print('Total number of symmetrical setting cases: ', diff)
    print('Symmetrical setting cases with un-identical resonance frequencies: ', diff0)
 
    sL1=sL1+' '+date_st+r'$-$'+date_end
    print('Maximum antenna currents: ', max(Ia_p), max(Ia_m))

# set axis, labels, color bar etc for left three panels
    axes.scatter(Ia_m, Ia_p, s=0.5, marker='.',color='red', edgecolors='none',\
        label=r'Symmetric setting: $f_{r+} = f_{r-}$')
    #if len(Ia_p0) > 0:
    #    axes.scatter(Ia_m0, Ia_p0, s=0.5, marker='.',color='black', \
    #        edgecolors='none', label=r'Symmetric setting: $f_{r+} \neq f_{r-}$')
    #if len(Ia_p01) > 0:
    #    axes.scatter(Ia_m01, Ia_p01, s=0.5, marker='.',color='blue', \
    #        edgecolors='none', label='Asymmetric setting')
    #if len(Ia_p2) > 0:
    #    axes.scatter(Ia_m2, Ia_p2, s=0.5, marker='.',color='black', \
    #        edgecolors='none', label='Undeterminable Q-factor')

    axes.legend(loc='upper center',fontsize=2.7,bbox_to_anchor=(0.25,0.99), \
            frameon=False)

    #axes.set_xlim(fmin,fmax)
    #axes.set_ylim(fmin,fmax)
    axes.tick_params(axis='both',which='major',length=1.3,width=0.4,pad=1, \
        labelsize=2.5,left=True,right=True,bottom=True,top=True,direction='in')
    axes.tick_params(axis='both',which='minor',length=0.8,width=0.4,pad=1, \
        labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes.xaxis.set_minor_locator(AutoMinorLocator())
    axes.yaxis.set_minor_locator(AutoMinorLocator())

    axes.set_xlabel (r'$I_{a-}$ (mA)', fontsize=2.7, labelpad=0.7)
    axes.set_ylabel (r'$I_{a+}$ (mA)', fontsize=2.7, labelpad=0.7)
    axes.text(0.01,1.01,sL1,fontsize=2.5,ha='left',va='bottom', \
        transform=axes.transAxes)

    return canv, fig, toolbar
