import numpy as np
import math
from scipy import stats
from scipy import interpolate

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

from scipy import interpolate

import moving_average

pi=math.pi
pi2=2*pi
c=3.0e8
eps0=8.85e-12
mu0=4.0e-7*pi

def Ra_Ca_balmain(fpe, fce, freq, La, raa):
    X=(fpe/freq)**2
    Y=fce/freq

    eps1=1.0-X/(1.0-Y*Y)
    eps3=1.0-X

    omega=pi2*freq

    #----- |a| parameter for Balmain impedance
    av=np.sqrt(abs(eps1/eps3))

    coef=av/(omega*pi2*eps0*eps1*La)

    # Ra for dipole = double of that for a monopole
    Rb=2.0*coef*(np.log(La/raa)-1.0-np.log(np.sqrt(1.0+av*av)/2.0))
    Xb=2.0*coef*np.arctan(av)

    return Rb, Xb

def Ra_wang_bell(fpe, fce, freq, La, raa):
    X=(fpe/freq)**2
    Y=fce/freq

    beta=pi2*freq/c
    Z0=377.0
    eps_0=1.0-X

    denom=Y*Y-1.0
    eps_s=X/denom
    alpha=Y/np.sqrt(denom)
        
    xi=(La*beta/alpha)*np.sqrt(eps_s)

    Rwb=Z0/(pi*alpha*np.sqrt(abs(eps_0))*xi) \
          *(np.log(2.0*La/(raa*alpha))-1.0-0.25*xi*xi+(Y+1)**1.5*xi**3/12.0)

    return Rwb

#Song et al formula for Ra
def Ra_song_coefficient(fpe, fce, freq):
    mu0=4.0e-7*math.pi
    c=3.0e8

    X=(fpe/freq)**2
    Y=fce/freq

    denom=1.0-Y*Y
    eps1=1.0-X/denom
    eps2=X*Y/denom
    eps3=1.0-X

    sinth2=1.0-4.0/Y**2
    q=(eps1**2-eps2**2-eps1*eps3)**2*sinth2**2+16.0*(eps2*eps3/Y)**2
    Cx=0.5*X*(X-eps3)*sinth2-X*(eps1*sinth2+4.0*eps3/Y**2)+eps1*eps3
    Cy=eps2*(X*sinth2-eps3)
    Ra_coef=48.0*pi**3*mu0*freq**2*fpe*(Cx*Cx+Cy*Cy)*sinth2/(c*fce*q)

    return Ra_coef

#------------------------------------------------------------------------------
# calculate normalized data points in each frequency - Xa or frequency - Ra bin
def number_data_points(N, ybin, fres, bwd, CaXa, Ra, corx):
    #-- for dot by dot plots of data points in each bin: y-axis
    y=np.zeros((2,ybin+1))
    yCX=np.zeros(ybin)
    yRa=np.zeros(ybin)

    fsmin=min(fres)
    fsmax=max(fres)

    xbin=int((fsmax-fsmin)/bwd)
    x=[0]*(xbin+1)
    xfr=[0]*xbin
    for k in range(xbin+1):
        x[k]=fsmin+float(k)*bwd
        if k > 0:
            xfr[k-1]=0.5*(x[k-1]+x[k])

    #--- determine y min & max
    CXmin=min(CaXa)
    CXmax=max(CaXa)
    Ramin=min(Ra)
    Ramax=max(Ra)
    dCX=(CXmax-CXmin)/ybin
    dRa=(Ramax-Ramin)/ybin

    #--- bin size and center of bins for Ca, Ra
    for k in range(ybin+1):
        y[0][k]=CXmin+float(k)*dCX
        y[1][k]=Ramin+float(k)*dRa
        if k > 0:
            yCX[k-1]=0.5*(y[0][k-1]+y[0][k])
            if corx == 'Xa':
                yCX[k-1]=10.0**yCX[k-1]

            yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
            yRa[k-1]=10.0**yRa[k-1]

    # --- calculate number of Ca data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,CaXa,fres,statistic='count', \
        bins=[x,y[0]])
    a=counts.statistic  # number of data points for Ca
    a=a.transpose()

    rdl=len(a[0])+1-N
    CXb2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        CXb2d[k]=moving_average.data_points_running_average(a[k], N)
    CXb2d=CXb2d.transpose()

    # --- calculate number of Ra data points in bins [x, y]
    counts=stats.binned_statistic_2d(fres,Ra,fres,statistic='count', \
        bins=[x,y[1]])
    a=counts.statistic  # number of data points for Ra
    a=a.transpose()

    rdl=len(a[0])+1-N
    Rab2d=np.zeros((len(a), rdl))
    for k in range(len(a)):
        Rab2d[k]=moving_average.data_points_running_average(a[k], N)
    Rab2d=Rab2d.transpose()

    return rdl, xfr, yCX, yRa, CXb2d, Rab2d
#------------------------------------------------------------------------------

#-------------------- plot Ra, Ca, and Prad only for Y+ plus Y- ---------------------------
def plot_RaCa_comparison(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, corx):

    pi=math.pi

    if inpflag !='In':
        print("Empirical electron density model only valid inside plasmasphere!")
        print('Change flag to inside the plasmasphere')
        inpflag='In'

    #datatpy=input('derived or song or balmain or wang? ')

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        datafile=fdir+'All-Bands_InOut_data-Nature.dat'
    else:
        datafile=fdir+'/All-Bands_InOut_data-Nature.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.2),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.10,0.10]
    yB=[0.60,0.26]
    wh=[0.52,0.52]
    ht=[0.32,0.32]

    txtsize=2.7

    axes=[]
    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

        #set line width of axes
        for ax in ['left','bottom','right','top']:
            axes[i].spines[ax].set_linewidth(0.3)

        axes[i].tick_params(axis='both',which='major',length=1.3,width=0.4, \
            pad=1,labelsize=txtsize,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 8080:
        sL1=sL1+r' $V_1 >= 80$ V'

    LL=[0.0375, 0.3015, 0.1885, 2.1515]

    date=[]
    ut=[]
    CaXa=[]
    Ra=[]
    fres=[]
    Iap=[]
    Iam=[]

    La=41.0
    raa=0.15e-3
    qe=1.6022e-19
    me=9.1e-31
    eps0=8.85e-12
    coef=4.0*(math.pi)**2*me*eps0/(qe*qe)

    fpe=400.0e3
    fce=100.0e3

    #Xa_song=[] #np.zeros(ndata)
    #Ra_song=[] #np.zeros(ndata)
    Xa_balm=[] #np.zeros(ndata)
    Ra_balm=[] #np.zeros(ndata)
    Ra_wang=[] #np.zeros(ndata)
    #fres_th=[]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpl != 1:
            print('Out of the plasmasphere:','{:2d}'.format(inpl)+' '+datet+' '+a[1])
            diff=diff+1
            continue

        fqp=float(a[5])
        fqm=float(a[16])
        Ra1=float(a[9])     #Ra in Ohm
        Ra2=float(a[20])

        if(L1 < 0.06 and (fqp>32.0e3 or fqm>32.0e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.3e3 or fqm>10.3e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 4e3 or fqm >4e3)):
            diff=diff+1
            continue

        Iap.append(float(a[8])*1.0e-3)  # Ia in A
        Iam.append(float(a[19])*1.0e-3)

        Ca1=float(a[11])     # antenna Ca in pF
        Ca2=float(a[22])

        date.append(datet)
        ut.append(float(a[1][0:2])*60.0+float(a[1][3:5])+float(a[1][6:8])/60.0) #Ut in min

        # measured antenna reactance two branches in Ohm
        Xa1=1.0e12/(pi2*fqp*Ca1)
        Xa2=1.0e12/(pi2*fqm*Ca2)

        if corx == 'Ca':
            CaXa.append(Ca1)               # Ca in pF
        else:
            CaXa.append(np.log10(Xa1+Xa2))     # logarithm of -Xa in Ohm

        Ra.append(np.log10(Ra1+Ra2))           #logarithm of Ra in Ohm

        fres.append(fqp/1.0e3)  # frequency as x-axis in kHz

    #-----------------------------------------------------------
    #end 'for loop' for input antenna parameters
    #-----------------------------------------------------------
    # --- calculate dipole antenna capacitance and resistance in vacuum
    fsmin=min(fres)
    fsmax=max(fres)

#-------------------------------------------------------------------
#-------------------------------------------------------------------
        # Song et al [2007] Xa & Song et al [2021] Ra. 
        # scale to 3-wire in parallel: Xa and Ra divided by 2.2

    fcurp=interpolate.interp1d(fres, Iap, fill_value="extrapolate")
    fcurm=interpolate.interp1d(fres, Iam, fill_value="extrapolate")

    df=np.zeros(3)
    freqmin=np.zeros(3)
    freqmin[0]=fsmin
    freqmin[1]=6.7
    freqmin[2]=22.6
    df[0]=(3.9-fsmin)/10.0
    df[1]=(10.2-freqmin[1])/10.0
    df[2]=10.0**((np.log10(fsmax)-np.log10(freqmin[2]))/10.0)

    fres_th=np.zeros(33)
    Xa_song=np.zeros(33)
    Ra_song=np.zeros(33)
    Xa_balm=np.zeros(33)
    Ra_balm=np.zeros(33)
    Ra_wang=np.zeros(33)

    for i in range(3):
      for j in range(11):
        fr=(freqmin[i]+df[i]*float(j))*1.0e3

        fres_th[i*11+j]=fr/1.0e3

        Ip=fcurp(fr/1.0e3)
        Im=fcurm(fr/1.0e3)

        Omega1=pi2*fr
        N0=coef*fpe**2
        aa=pi*Omega1*La
        Xa_s= 1.0/(4.0*aa*eps0)*(np.log(Ip/(aa*qe*N0*raa*raa)+2.0)-1.0)/3 \
             +1.0/(4.0*aa*eps0)*(np.log(Im/(aa*qe*N0*raa*raa)+2.0)-1.0)/3
        Ra_s=3.9e-13*fce**3*fpe*La*La/(fr**2)
        #Ra_s=Ra_song_coefficient(fpe, fce, fqp)*La**
        Xa_song[i*11+j]=Xa_s
        Ra_song[i*11+j]=Ra_s

        # Bailmain [1964] Xa and Ra of dipole
        Ra_b, Xa_b=Ra_Ca_balmain(fpe, fce, fr, La, raa)
        Xa_balm[i*11+j]=Xa_b*3000  #multiply Xa_b by 3000 but not scaled by 2.2
        Ra_balm[i*11+j]=Ra_b

        # Wang & Bell 1969 Ra
        Ra_wang[i*11+j]=Ra_wang_bell(fpe, fce, fr, La, raa)

    Ra_wang[0:22]=Ra_wang[23]
    Ra_wang[30:33]=Ra_wang[29]
    Ra_balm[30:33]=Ra_balm[29]

    xbin=int(fsmax+3-fsmin)
    freq=[0]*xbin
    Rvac=[0]*xbin
    Cvac=[0]*xbin
    Xvac=[0]*xbin

    La=80.0
    raa=0.15e-3
    for j in range(xbin):
        freq[j]=fsmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        Omega=pi2*freq[j]*1.0e3
        Rvac[j]=20.0*(pi*La/wlen)**2      # vacuum dipole resistance
        Xvac[j]=120.0*(np.log(La/(2.0*raa))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=1.0e12/(Omega*Xvac[j])    # vacuum dipole Capacitance

        #dipole Ca, Xa, and Ra for 3 wires
        Cvac[j]=Cvac[j]*3.0
        Xvac[j]=Xvac[j]/2.2
        Rvac[j]=Rvac[j]*4.0e5

    print('Min & max vacuum Ca:',("{:8>.2f}"*2).format(min(Cvac),max(Cvac)))

    if diff > 0:
        print('\n',diff,'data out of',nrecd,'not used. Used data: ', nrecd-diff)

    if bname == 'All-Bands':
        xscale='log'
        if corx == 'Ca':
            yscale='linear'
        else:
            yscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        yscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        yscale='linear'
        fmin=min(fres)
        fmax=max(fres)

    #---------------------------------------------------------------------------------
    N=8   # running average window width
    ybin=100 # for 2D binning not for 1D

    # plot number of derived, or Song et al or Balmain [19640 data points in each bin, 
    # onece for yCa (or yRa or ypw) points at given frequency xfr[j]. The data points 
    # number is normalized by maximum at each frequency
    #
    #--------- Xa and Ra in log10 freq in kHz ----------------------------------------
    rdl, xfr, yCX, yRa, CXb2d, Rab2d = number_data_points(N, ybin, fres, bwd, \
        CaXa, Ra, corx)

    #--- plot scatter data points at frequency xfr[j] (kHz): the center of frequency bin
    sc=[0]*2
    for j in range(len(xfr[0:rdl])):
        xx=[xfr[j]]*ybin

        CXbmax=max(CXb2d[j])  #maximum number of data points in this frequency bin
        if CXbmax > 0:
            sc[0]=axes[0].scatter(xx,yCX,c=CXb2d[j]/CXbmax,cmap=colormp,s=0.1,marker='s')

        Rabmax=max(Rab2d[j])
        if Rabmax > 0:
            sc[1]=axes[1].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1,marker='s')

    #---------------------------------------------------------------------------------------

    #if datatpy == 'wang':
    axes[0].plot(fres_th, Xa_balm, lw=0.5, ls='dashed', color='black',label=r'Balmain [1964]')
    axes[1].plot(fres_th, Ra_balm, lw=0.5, ls='dashed', color='black',label='Balmain [1964]')
    axes[1].plot(fres_th, Ra_wang, lw=0.5, color='black',label='Wang & Bell [1969]')

#    axes[1].plot(fres_th, Ra_wang, s=0.04, marker='+',color='black', edgecolors=None, \
#            label='Wang & Bell [1969]')
#    axes[0].plot(fres_th, Xa_balm, s=0.1, marker='+',color='blue', edgecolors=None, \
#        label=r'Balmain [1964] $X_a \times 3000$')
#    axes[1].plot(fres_th, Ra_balm, s=0.1, marker='+',color='blue', edgecolors=None, \
#        label='Balmain [1964]')

    #----- scale Tu et al [2008] reactance & resistance to DSX
    Ld=41.0 #one branch length of DSX antenna
    LT=50   #one branch length in Tu et al [2008]
    rT=0.2
    NT=500.0e6
    IT=0.4
    Nd=4.0*(math.pi*fpe)**2*0.000314589843
    fTu=[30.0]
    Omega=pi2*fTu[0]*1.0e3
    #CXTu=[3486.2/2.0] # one branch reactance
    #a1=(np.exp(4.0*pi*Omega*eps0*LT*CXTu[0]+1.0)-2.0)*LT*rT*rT*NT
    #scaled to DSX reactance for two branches/3 wires in Ohm
    #CXTu[0]=2.0/3.0*CXTu[0]*LT/Ld*(np.log(a1/(Nd*Ld*raa*raa)+2.0)-1.0) \
    #        /(np.log(a1/(NT*LT*rT*rT)+2.0)-1.0)
    #CXTu=[3486.2/3.0*100.0/La*(1.0+np.log(100.0/La*(0.2/raa)**2))]
    pi=math.pi
    cof=pi*pi*fTu[0]*1.0e3*1.06e-19
    bTu=np.log(IT/(cof*LT*NT*rT**2)+1)-1
    bDSX=np.log(40.0e-3/(cof*Ld*Nd*raa**2)+1)-1
    CXTu=[3486.2*100.0/82*bDSX/bTu]
    #RaTu=[CXTu[0]/np.tan(88.59*pi/180.0)]
    fpeTu=1.0/(2.0*pi)*np.sqrt(1.6**2*1.0e-38*NT/(9.1e-31*8.85e-12))
    RaS=2789.2/0.8*np.cos(88.59*pi/180.0)
    RaTu=[RaS*fpe/fpeTu*(Ld/LT)**2]
    #----- completed scaling Tu et al reactance & resistance

    #if datatpy=='song':

    fChe=[2.55,2.75,3.05,3.20,3.50,3.92,4.36,4.85,5.42,5.90,6.5,7.0,7.5,8.0, \
            8.5,9.0,9.5,10.0]
    fWang=[2.55,2.75,3.05,3.20,3.50,3.92,4.36,4.85,5.42,5.90,6.5,7.0,7.5,8.0, \
            8.5,9.0,9.5,10.0, 33.0]
    CXWang=[0]*19
    CXChe=[0]*18
    for j in range(18):
        CXWang[j]=3e3*(0.7*fChe[j]+0.8557)
        CXChe[j]=3e3*(0.73*fWang[j]+0.8557)
    CXWang[18]=3e3*(0.7*fWang[18]+0.8557)

    RaChe=[1490.0,1080,808.0,725.0,655.0,610.0,575.0, \
            550.0,520.0,510.0,505.0,500.0,500.0,500.0, \
            500.0,500.0,500.0,500.0]
    RaWang=[1850.0,1150,828.0,735.0,660.0,615.0,580.0, \
            550.0,520.0,510.0,505.0,500.0,500.0,500.0, \
            500.0,500.0,500.0,500.0]

    axes[0].plot(fWang,CXWang, lw=0.5, color='black', label='Wang [1970]')
    axes[0].plot(freq, Xvac, lw=0.5, color='blue', ls='dashed', label='Dipole in vacuum')
    axes[0].scatter(fres_th[0:32], Xa_song[0:32], s=0.1, marker='+',color='black', edgecolors=None, \
        label='Song et al [2007]')
    #axes[0].scatter(fTu,CXTu, s=1, marker='^', color='black', label='Tu et al [2008]')
    #axes[0].scatter(fChe,CXChe, s=0.2, marker='o', color='red',label='Chevalier et al [2008]')
    axes[0].text(0.2,0.13,r'Multiplied by 3000',fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes,color='black',rotation=13)

    #axes[1].plot(fChe,RaWang, lw=0.5, color='black', label='Wang & Bell [1972]')
    axes[1].plot(freq, Rvac, lw=0.5, color='blue', ls='dashed', label='Dipole in vacuum')
    axes[1].scatter(fres_th[0:32], Ra_song[0:32], s=0.1, marker='+',color='black', edgecolors=None, \
        label='Song et al [2022]')
    #axes[1].scatter(fTu,RaTu, s=1, marker='^', color='black', label='Tu et al [2008]')
    #axes[1].scatter(fChe,RaChe, s=0.2, marker='o', color='red',label='Chevalier et al [2008]')
    axes[1].text(0.3,0.05,r'Multiplied by 40000',fontsize=3,ha='center',va='bottom', \
        transform=axes[1].transAxes,color='black',rotation=19)

    legd=axes[0].legend(loc='upper center',fontsize=2.2,bbox_to_anchor=(0.75,0.99), \
            frameon=True)
    legd=axes[1].legend(loc='upper center',fontsize=2.2,bbox_to_anchor=(0.70,0.99), \
            frameon=True)

    # --- y-axis label
    if corx == 'Ca':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=txtsize,labelpad=0.7)
    else:
        axes[0].set_ylabel('$-X_a$ (Ohm)',fontsize=txtsize,labelpad=0.7)

    #----- set axis properties for Ca or Xa------
    axes[0].set_xlim(fmin,fmax)
    #axes[0].set_ylim(1e4,4e5)
    axes[0].set_xscale(xscale)
    axes[0].set_yscale(yscale)
    axes[0].set_xticklabels([])

    axes[1].set_xlim(fmin,fmax)
    axes[1].set_ylim(30,8e4)
    axes[1].set_xscale(xscale)
    axes[1].set_yscale('log')
    axes[1].set_ylabel('$R_a$ (Ohm)',fontsize=txtsize,labelpad=0.7)
    axes[1].set_xlabel('Resonance Frequency (kHz)',fontsize=txtsize, labelpad=0.7)

    axes[1].set_xticks([2.5,5,10,20,30,40])
    axes[1].set_xticklabels(['2.5','5','10','20','30','40'])

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    txt=sL1+' Inside Nominal Plasmasphere'

    xloc=0.5
    axes[0].text(xloc,1.09,datemin[0:10]+'$-$'+datemax[0:10],fontsize=txtsize, \
        ha='center',va='bottom',transform=axes[0].transAxes)

    axes[0].text(xloc,1.01,txt,fontsize=txtsize,ha='center',va='bottom', \
        transform=axes[0].transAxes)

        #for text in legd.get_texts():
        #    text.set_color('blue')

    # ---- color bar ----
    xloc=0.085
    axins = inset_axes(axes[1], width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(xloc,-0.36,1,1), bbox_transform=axes[1].transAxes)
    cbar=fig.colorbar(sc[1], cax=axins, shrink=0.9, pad=0.04, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=2.1, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.3, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar
