import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import days_of_year
import binned_values
import moving_average

#----------- plot one antenna parameter for Y + In vs Out plasmasphere ------
def plot_one_InvsOut(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    ptype, V0, fl, comb):
    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.10,0.52]
    yB=[0.50,0.50]
    wh=[0.38,0.38]
    ht=[0.35,0.35]

    axes=[]
    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(2):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname
    if comb == 'Yes':
        sL1=sL1+' 1st Harmonic'
    else:
        sL1=sL1+' Y+ 1st Harmonic'
    if V0 < 262:
        sL1=sL1+' V1 = '+str(V0)+' V'

    if bname == 'All-Bands':
        if comb == 'Yes':
            minCa=200.0
            maxCa=450.0
            minRa=0.06
            maxRa=100
            minPr=0.2
            maxPr=100.0
            mineff=0.0
            maxeff=1.0
        else:
            minCa=400.0
            maxCa=900.0
            minRa=0.03
            maxRa=40
            minPr=0.1
            maxPr=50.0
            mineff=0.0
            maxeff=1.0
    elif bname == 'High-Band':
        minCa=400.0
        maxCa=700.0
        minRa=0.1
        maxRa=2.0
        minPr=0.5
        maxPr=50.0
    elif bname == 'Mid-Band':
        minCa=400.0
        maxCa=850.0
        minRa=0.2
        maxRa=10.0
        minPr=0.2
        maxPr=20.0
    elif bname == 'Low-Band':
        minCa=450
        maxCa=900.0
        minRa=0.3
        maxRa=40.0
        minPr=0.1
        maxPr=6.0

    date=[]
    fres=[[],[]]
    Ca=[[],[]]
    Ra=[[],[]]
    pw=[[],[]]
    eff=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        date.append(datet)
        V1=int(a[2])
        L1=float(a[3])
        Va=float(a[6])
        inpl=int(a[24])

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L) \
                    +' '+datet+' '+a[1])
                diff=diff+1
                continue

        if V0 < 262 and V1 != V0:
            print('Exclude data with different driving voltage:', V0)
            diff=diff+1
            continue

        if inpl == 1:
            fq1=float(a[5])
            Ra1=float(a[9])        #Ra in k Ohm
            Ca1=float(a[11])       #Ca in pF

            if(L1 < 0.06 and fq1>30.1e3):
                diff=diff+1
                continue
            if ((L1 > 0.1 and L1 < 0.5) and fq1>10.2e3):
                diff=diff+1
                continue
            if (L1 > 1.0 and fq1 > 3.9e3):
                diff=diff+1
                continue

        # ---- Calculate power supplied at the resonance --------------------------
            # 1/2*Itotal^2*Ra*(Ca/Ct)^2 = 1/2*Va^2*Ra/(Ra^2+1/(Omega*Ca)^2)
            Ct1=float(a[4])+Ca1+float(a[12])
            Omega=2.0*math.pi*fq1
            Itot1=(Ct1/Ca1)*Va/np.sqrt((Ra1*Ra1+1.0/(Omega*Ca1*1.0e-12)**2))

            if comb == 'Yes':
                fq2=float(a[16])
                Ra2=float(a[20])       #Ra in k Ohm
                Ca2=float(a[22])       #Ca in pF

                Ct2=float(a[15])+Ca2+float(a[23])
                Omega=2.0*math.pi*fq2
                Itot2=(Ct2/Ca2)*float(a[17])/np.sqrt((Ra2*Ra2+1.0/(Omega*Ca2*1.0e-12)**2))
                prad1=0.5e-6*(float(a[8])**2*Ra1+float(a[19])**2*Ra2)
                Psupp=0.5*(Itot1**2*float(a[10])+Itot2**2*float(a[21]))+prad1

                fq1=0.5*(fq1+fq2)
                Ra1=Ra1+Ra2
                Ca1=Ca1*Ca2/(Ca1+Ca2)
            else:
                prad1=0.5e-6*float(a[8])**2*Ra1   #(I in mA. Pwer in W)
                Psupp=0.5*Itot1**2*float(a[10])+prad1

            fres[0].append(fq1/1.0e3)         #frequency in kHz
            Ca[0].append(Ca1)
            Ra[0].append(np.log10(Ra1/1.0e3)) #log scale in kilo Ohm
            pw[0].append(np.log10(prad1))    #logscale in W
            eff[0].append(prad1/Psupp)
        elif inpl == 2:
            fq2=float(a[16])
            Ra2=float(a[20])      #Ra in k Ohm
            Ca2=float(a[22])      #Ca in pF

        # ---- Calculate power supplied at the resonance --------------------------
            # 1/2*Itotal^2*Ra*(Ca/Ct)^2 = 1/2*Va^2*Ra/(Ra^2+1/(Omega*Ca)^2)
            Ct2=float(a[15])+Ca2+float(a[23])
            Omega=2.0*math.pi*fq2
            Itot2=(Ct2/Ca2)*float(a[17])/np.sqrt((Ra2*Ra2+1.0/(Omega*Ca2*1.0e-12)**2))

            if comb == 'Yes':
                fq1=float(a[5])
                Ra1=float(a[9])       #Ra in k Ohm
                Ca1=float(a[11])      #Ca in pF

                Ct1=float(a[4])+Ca1+float(a[12])
                Omega=2.0*math.pi*fq1
                Itot1=(Ct1/Ca1)*Va/np.sqrt((Ra1*Ra1+1.0/(Omega*Ca1*1.0e-12)**2))
                prad2=0.5e-6*(float(a[8])**2*Ra1+float(a[19])**2*Ra2)
                Psupp=0.5*(Itot1**2*float(a[10])+Itot2**2*float(a[21]))+prad2

                fq2=0.5*(fq1+fq2)
                Ra2=Ra1+Ra2
                Ca2=Ca1*Ca2/(Ca1+Ca2)
            else:
                prad2=0.5e-6*float(a[19])**2*Ra2   #(I in mA. Pwer in W)
                Psupp=0.5*Itot2**2*float(a[21])+prad2

            fres[1].append(fq2/1.0e3)
            Ca[1].append(Ca2)
            Ra[1].append(np.log10(Ra2/1.0e3))
            pw[1].append(np.log10(prad2))
            eff[1].append(prad2/Psupp)

    #end 'for loop' for reading data file

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    # --- calculate dipole antenna capacitance
    pi=math.pi
    pi2=2.0*pi
    La=80.0
    ra=0.15e-3

    xbin=int(fmax+2-fmin)
    freq=[0]*xbin
    Cvac=[0]*xbin

    for j in range(xbin):
        freq[j]=fmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        omega=pi2*freq[j]*1.0e3
        Xvac=120.0*(np.log(La/(2.0*ra))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=1.0e12/(omega*Xvac)       # vacuum dipole Capacitance
        if comb == 'No':
            Cvac[j]=2.0*Cvac[j]           #double vacuum dipole Ca for one element

    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=100 # for 2D binning not for 1D
    y=np.zeros((4, ybin+1))
    yCa=np.zeros(ybin)
    yRa=np.zeros(ybin)
    ypw=np.zeros(ybin)
    yeff=np.zeros(ybin)

    for i in range(2):
        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Camin=min(Ca[i])
        Camax=max(Ca[i])
        dCa=(Camax-Camin)/ybin

        Ramin=min(Ra[i])
        Ramax=max(Ra[i])
        dRa=(Ramax-Ramin)/ybin

        pwmin=min(pw[i])
        pwmax=max(pw[i])
        dpw=(pwmax-pwmin)/ybin

        effmin=min(eff[i])
        effmax=max(eff[i])
        deff=(effmax-effmin)/ybin

    #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[0][k]=Camin+float(k)*dCa
            y[1][k]=Ramin+float(k)*dRa
            y[2][k]=pwmin+float(k)*dpw
            y[3][k]=effmin+float(k)*deff
            if k > 0:
                yCa[k-1]=0.5*(y[0][k-1]+y[0][k])

                yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
                yRa[k-1]=10.0**yRa[k-1]

                ypw[k-1]=0.5*(y[2][k-1]+y[2][k])
                ypw[k-1]=10.0**ypw[k-1]

                yeff[k-1]=0.5*(y[3][k-1]+y[3][k])

        if ptype == 'Ca_Only':
            # --- calculate number of Ca data points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],Ca[i],fres[i],statistic='count', \
                bins=[x,y[0]])
            a=counts.statistic  # number of data points for Ca
            a=a.transpose()

            rdl=len(a[0])+1-N
            Cab2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                Cab2d[k]=moving_average.data_points_running_average(a[k], N)
            Cab2d=Cab2d.transpose()

            # plot number of data points in each bin, onece for yCa
            # points at given frequency xfr[j]. The data points number is normalized
            # by maximum at each frequency
            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                Cabmax=max(Cab2d[j])

                if Cabmax > 0:
                    sc=axes[i].scatter(xx,yCa,c=Cab2d[j]/Cabmax,cmap=colormp,s=0.1, \
                        marker='s')

            #----- set axis properties for Ca ------
            axes[i].plot(freq, Cvac, ls='dashed', lw=0.3, color='black')
            axes[i].set_ylim(minCa,maxCa)
            axes[i].yaxis.set_minor_locator(AutoMinorLocator())

        elif ptype == 'Ra_Only':
            # --- calculate number of Ca data points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],Ra[i],fres[i],statistic='count', \
                bins=[x,y[1]])
            a=counts.statistic  # number of data points for Ca
            a=a.transpose()

            rdl=len(a[0])+1-N
            Rab2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                Rab2d[k]=moving_average.data_points_running_average(a[k], N)
            Rab2d=Rab2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                Rabmax=max(Rab2d[j])
                if Rabmax > 0:
                    sc=axes[i].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(minRa,maxRa)
            axes[i].set_yscale('log')

        elif ptype == 'Pa_Only':
        # --- calculate number of power points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],pw[i],fres[i],statistic='count', \
                bins=[x,y[2]])
            a=counts.statistic  # number of data points for pwr
            a=a.transpose()

            rdl=len(a[0])+1-N
            pwb2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                pwb2d[k]=moving_average.data_points_running_average(a[k], N)
            pwb2d=pwb2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                pwbmax=max(pwb2d[j])
                if pwbmax > 0:
                    sc=axes[i].scatter(xx,ypw,c=pwb2d[j]/pwbmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(minPr,maxPr)
            axes[i].set_yscale('log')

        elif ptype == 'Eff_Only':
        # --- calculate number of power points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],eff[i],fres[i],statistic='count', \
                bins=[x,y[3]])
            a=counts.statistic  # number of data points for pwr
            a=a.transpose()

            rdl=len(a[0])+1-N
            effb2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                effb2d[k]=moving_average.data_points_running_average(a[k], N)
            effb2d=effb2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                effbmax=max(effb2d[j])
                if effbmax > 0:
                    sc=axes[i].scatter(xx,yeff,c=effb2d[j]/effbmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(mineff,maxeff)

        axes[i].set_xlim(fmin,fmax)
        axes[i].set_xscale(xscale)
        if bname == 'All-Bands':
            axes[i].set_xticks([2.5,5,10,20,30,40])
            axes[i].set_xticklabels(['2.5','5','10','20','30','40'])

        axes[i].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
        axes[i].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='out')

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)

    if ptype == 'Ca_Only':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.8)
    elif ptype == 'Ra_Only':
        axes[0].set_ylabel('$R_a$ ($10^{3}$ Ohm)',fontsize=2.5,labelpad=0.8)
    elif ptype == 'Pa_Only':
        axes[0].set_ylabel('Power Dissipated (W)',fontsize=2.5,labelpad=0.8)
    else:
        axes[0].set_ylabel('Efficiency',fontsize=2.5,labelpad=0.8)

    axes[1].set_yticklabels([])
    axes[0].text(1,1.11,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1,1.04,sL1,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Inside Plasmasphere',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
    axes[1].text(1,1.04,'Outside Plasmasphere',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[1].transAxes)

    # ---- color bar ----
    axins = inset_axes(axes[0], width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(0.61,-0.36,1,1), bbox_transform=axes[0].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=2.2, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar, fl

#----------- plot one antenna parameters for Y+ vs Y- -----------------
def plot_one_InOut(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    ptype, V0, fl):

    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.4,1.8),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.10,0.52]
    yB=[0.50,0.50]
    wh=[0.34,0.34]
    ht=[0.35,0.35]

    axes=[]
    for i in range(2):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(2):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname
    sL1=sL1+' and All Regions'
    if V0 < 262:
        sL1=sL1+' V1 = '+str(V0)+' V'

    if bname == 'All-Bands':
        minCa=400.0
        maxCa=900.0
        minRa=0.03
        maxRa=40
        minPr=0.1
        maxPr=50.0
        mineff=0.0
        maxeff=1.0
    elif bname == 'High-Band':
        minCa=400.0
        maxCa=700.0
        minRa=0.1
        maxRa=2.0
        minPr=0.5
        maxPr=50.0
    elif bname == 'Mid-Band':
        minCa=400.0
        maxCa=850.0
        minRa=0.2
        maxRa=10.0
        minPr=0.2
        maxPr=20.0
    elif bname == 'Low-Band':
        minCa=450
        maxCa=900.0
        minRa=0.3
        maxRa=40.0
        minPr=0.1
        maxPr=6.0

    date=[]
    fres=[[],[]]
    Ca=[[],[]]
    Ra=[[],[]]
    pw=[[],[]]
    eff=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        V1=float(a[2])
        date.append(datet)
        L1=float(a[3])

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L) \
                    +' '+datet+' '+a[1])
                diff=diff+1
                continue

        if V0 < 262 and V1 != V0:
            print('Exclude data with different driving voltage:', V0)
            diff=diff+1
            continue

        fq1=float(a[5])
        Ca1=float(a[11])       #Ca in pF
        Ra1=float(a[9])        #Ra in k Ohm
        prad1=0.5e-6*float(a[8])**2*Ra1   #(I in mA. Pwer in W)
        # ---- Calculate power supplied at the resonance --------------------------
        # 1/2*Itotal^2*Ra*(Ca/Ct)^2 = 1/2*Va^2*Ra/(Ra^2+1/(Omega*Ca)^2)
        Ct=float(a[4])+Ca1+float(a[12])
        Va=float(a[6])
        Omega=2.0*math.pi*fq1
        Itot=(Ct/Ca1)*Va/np.sqrt((Ra1*Ra1+1.0/(Omega*Ca1*1.0e-12)**2))
        Pinp=0.5*Itot**2*float(a[10])+prad1

        fres[0].append(fq1/1.0e3)         #frequency in kHz
        Ca[0].append(Ca1)
        Ra[0].append(np.log10(Ra1/1.0e3)) #log scale in kilo Ohm
        pw[0].append(np.log10(prad1))    #logscale in W
        eff[0].append(prad1/Pinp)

        fq2=float(a[16])
        Ca2=float(a[22])       #Ca in pF
        Ra2=float(a[20])        #Ra in k Ohm
        prad2=0.5e-6*float(a[19])**2*Ra2   #(I in mA. Pwer in W)
        # ---- Calculate power supplied at the resonance --------------------------
        # 1/2*Itotal^2*Ra*(Ca/Ct)^2 = 1/2*Va^2*Ra/(Ra^2+1/(Omega*Ca)^2)
        Ct=float(a[15])+Ca2+float(a[23])
        Va=float(a[17])
        Omega=2.0*math.pi*fq2
        Itot=(Ct/Ca2)*Va/np.sqrt((Ra2*Ra2+1.0/(Omega*Ca2*1.0e-12)**2))
        Pinp=0.5*Itot**2*float(a[21])+prad2
            
        fres[1].append(fq2/1.0e3)
        Ca[1].append(Ca2)
        Ra[1].append(np.log10(Ra2/1.0e3))
        pw[1].append(np.log10(prad2))
        eff[1].append(prad2/Pinp)

    #end 'for loop' for reading data file

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    # --- calculate dipole antenna capacitance
    pi=math.pi
    pi2=2.0*pi
    La=80.0
    ra=0.15e-3

    xbin=int(fmax+2-fmin)
    freq=[0]*xbin
    Cvac=[0]*xbin

    for j in range(xbin):
        freq[j]=fmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        omega=pi2*freq[j]*1.0e3
        Xvac=120.0*(np.log(La/(2.0*ra))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=2.0e12/(omega*Xvac)       # double of vacuum dipole Capacitance

    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=50 # for 2D binning not for 1D
    y=np.zeros((4, ybin+1))
    yCa=np.zeros(ybin)
    yRa=np.zeros(ybin)
    ypw=np.zeros(ybin)
    yeff=np.zeros(ybin)

    for i in range(2):
        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Camin=min(Ca[i])
        Camax=max(Ca[i])
        dCa=(Camax-Camin)/ybin

        Ramin=min(Ra[i])
        Ramax=max(Ra[i])
        dRa=(Ramax-Ramin)/ybin

        pwmin=min(pw[i])
        pwmax=max(pw[i])
        dpw=(pwmax-pwmin)/ybin

        effmin=min(eff[i])
        effmax=max(eff[i])
        deff=(effmax-effmin)/ybin

        #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[0][k]=Camin+float(k)*dCa
            y[1][k]=Ramin+float(k)*dRa
            y[2][k]=pwmin+float(k)*dpw
            y[3][k]=effmin+float(k)*deff
            if k > 0:
                yCa[k-1]=0.5*(y[0][k-1]+y[0][k])

            if k > 0:
                yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
                yRa[k-1]=10.0**yRa[k-1]

                ypw[k-1]=0.5*(y[2][k-1]+y[2][k])
                ypw[k-1]=10.0**ypw[k-1]

                yeff[k-1]=0.5*(y[3][k-1]+y[3][k])
                #yeff[k-1]=10.0**yeff[k-1]

        if ptype == 'Ca_Only':
            # --- calculate number of Ca data points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],Ca[i],fres[i],statistic='count', \
                bins=[x,y[0]])
            a=counts.statistic  # number of data points for Ca
            a=a.transpose()

            rdl=len(a[0])+1-N
            Cab2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                Cab2d[k]=moving_average.data_points_running_average(a[k], N)
            Cab2d=Cab2d.transpose()

            # plot number of data points in each bin, onece for yCa
            # points at given frequency xfr[j]. The data points number is normalized
            # by maximum at each frequency
            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                Cabmax=max(Cab2d[j])
                if Cabmax > 0:
                    sc=axes[i].scatter(xx,yCa,c=Cab2d[j]/Cabmax,cmap=colormp,s=0.1, \
                        marker='s')

            #----- set axis properties for Ca ------
            axes[i].plot(freq, Cvac, ls='dashed', lw=0.3, color='black')
            axes[i].set_ylim(minCa,maxCa)
            axes[i].yaxis.set_minor_locator(AutoMinorLocator())

        elif ptype == 'Ra_Only':
            # --- calculate number of Ca data points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],Ra[i],fres[i],statistic='count', \
                bins=[x,y[1]])
            a=counts.statistic  # number of data points for Ca
            a=a.transpose()

            rdl=len(a[0])+1-N
            Rab2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                Rab2d[k]=moving_average.data_points_running_average(a[k], N)
            Rab2d=Rab2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                Rabmax=max(Rab2d[j])
                if Rabmax > 0:
                    sc=axes[i].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(minRa,maxRa)
            axes[i].set_yscale('log')

        elif ptype == 'Pa_Only':
        # --- calculate number of power points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],pw[i],fres[i],statistic='count', \
                bins=[x,y[2]])
            a=counts.statistic  # number of data points for pwr
            a=a.transpose()

            rdl=len(a[0])+1-N
            pwb2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                pwb2d[k]=moving_average.data_points_running_average(a[k], N)
            pwb2d=pwb2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                pwbmax=max(pwb2d[j])
                if pwbmax > 0:
                    sc=axes[i].scatter(xx,ypw,c=pwb2d[j]/pwbmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(minPr,maxPr)
            axes[i].set_yscale('log')

        elif ptype == 'Eff_Only':
        # --- calculate number of power points in bins [x, y]
            counts=stats.binned_statistic_2d(fres[i],eff[i],fres[i],statistic='count', \
                bins=[x,y[3]])
            a=counts.statistic  # number of data points for pwr
            a=a.transpose()

            rdl=len(a[0])+1-N
            effb2d=np.zeros((len(a), rdl))
            for k in range(len(a)):
                effb2d[k]=moving_average.data_points_running_average(a[k], N)
            effb2d=effb2d.transpose()

            for j in range(len(xfr[0:rdl])):
                xx=[xfr[j]]*ybin
                effbmax=max(effb2d[j])
                if effbmax > 0:
                    sc=axes[i].scatter(xx,yeff,c=effb2d[j]/effbmax,cmap=colormp,s=0.1, \
                        marker='s')

            axes[i].set_ylim(mineff,maxeff)

        axes[i].set_xlim(fmin,fmax)
        axes[i].set_xscale(xscale)
        if bname == 'All-Bands':
            axes[i].set_xticks([2.5,5,10,20,30,40])
            axes[i].set_xticklabels(['2.5','5','10','20','30','40'])

        axes[i].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
        axes[i].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='out')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='out')

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)

    if ptype == 'Ca_Only':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.8)
    elif ptype == 'Ra_Only':
        axes[0].set_ylabel('$R_a$ ($10^{3}$ Ohm)',fontsize=2.5,labelpad=0.8)
    elif ptype == 'Pa_Only':
        axes[0].set_ylabel('Power Dissipated (W)',fontsize=2.5,labelpad=0.8)
    else:
        axes[0].set_ylabel('Efficiency',fontsize=2.5,labelpad=0.8)

    axes[1].set_yticklabels([])
    axes[0].text(1,1.11,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1,1.04,sL1,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
    axes[1].text(1,1.04,'Y- 1st Harmonic',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[1].transAxes)

    # ---- color bar ----
    axins = inset_axes(axes[0], width='70%', height='7%', loc='lower left', \
    bbox_to_anchor=(0.61,-0.36,1,1), bbox_transform=axes[0].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=2.2, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar, fl
